import itertools
import os
import unittest
from collections import defaultdict
from pathlib import Path


from typing import Dict, Any

import mock
from parameterized import parameterized_class

# do not remove - prevents circular import
from checkov.common.bridgecrew.check_type import CheckType
from checkov.common.bridgecrew.severities import BcSeverities, Severities
from checkov.common.graph.db_connectors.igraph.igraph_db_connector import IgraphConnector
from checkov.common.graph.db_connectors.networkx.networkx_db_connector import NetworkxConnector
from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.runner_filter import RunnerFilter
from checkov.terraform import TFDefinitionKey
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck
from checkov.terraform.plan_runner import Runner, resource_registry


@parameterized_class([
   {"db_connector": NetworkxConnector},
   {"db_connector": IgraphConnector}
])
class TestRunnerValid(unittest.TestCase):

    def setUp(self) -> None:
        self.orig_checks = resource_registry.checks
        self.db_connector = self.db_connector

    def test_runner_two_checks_only(self):
        current_dir = os.path.dirname(os.path.realpath(__file__))
        valid_plan_path = current_dir + "/resources/plan/tfplan.json"
        runner = Runner(db_connector=self.db_connector())
        checks_allowlist = ["CKV_AWS_21"]
        report = runner.run(
            root_folder=None,
            files=[valid_plan_path],
            external_checks_dir=None,
            runner_filter=RunnerFilter(framework=["all"], checks=checks_allowlist),
        )
        report_json = report.get_json()
        self.assertIsInstance(report_json, str)
        self.assertIsNotNone(report_json)
        self.assertIsNotNone(report.get_test_suite())
        self.assertEqual(report.get_exit_code({'soft_fail': False, 'soft_fail_checks': [], 'soft_fail_threshold': None, 'hard_fail_checks': [], 'hard_fail_threshold': None}), 1)
        self.assertEqual(report.get_exit_code({'soft_fail': True, 'soft_fail_checks': [], 'soft_fail_threshold': None, 'hard_fail_checks': [], 'hard_fail_threshold': None}), 0)

        for record in report.failed_checks:
            self.assertIn(record.check_id, checks_allowlist)
        self.assertEqual(report.get_summary()["failed"], 3)
        self.assertEqual(report.get_summary()["passed"], 3)

    def test_runner_record_severity(self):
        current_dir = os.path.dirname(os.path.realpath(__file__))
        valid_plan_path = current_dir + "/resources/plan/tfplan.json"
        runner = Runner()

        custom_check_id = "MY_CUSTOM_CHECK"

        resource_registry.checks = defaultdict(list)

        class AnyFailingCheck(BaseResourceCheck):
            def __init__(self, *_, **__) -> None:
                super().__init__(
                    "this should fail",
                    custom_check_id,
                    [CheckCategories.ENCRYPTION],
                    ["aws_db_instance"]
                )

            def scan_resource_conf(self, conf: Dict[str, Any]) -> CheckResult:
                return CheckResult.FAILED

        check = AnyFailingCheck()
        check.severity = Severities[BcSeverities.LOW]
        checks_allowlist = [custom_check_id]
        report = runner.run(
            root_folder=None,
            files=[valid_plan_path],
            external_checks_dir=None,
            runner_filter=RunnerFilter(framework=["terraform_plan"], checks=checks_allowlist),
        )

        self.assertEqual(report.failed_checks[0].severity, Severities[BcSeverities.LOW])

    def test_runner_check_severity_filter_omit(self):
        current_dir = os.path.dirname(os.path.realpath(__file__))
        valid_plan_path = current_dir + "/resources/plan/tfplan.json"
        runner = Runner()

        custom_check_id = "MY_CUSTOM_CHECK"

        resource_registry.checks = defaultdict(list)

        class AnyFailingCheck(BaseResourceCheck):
            def __init__(self, *_, **__) -> None:
                super().__init__(
                    "this should fail",
                    custom_check_id,
                    [CheckCategories.ENCRYPTION],
                    ["aws_db_instance"]
                )

            def scan_resource_conf(self, conf: Dict[str, Any]) -> CheckResult:
                return CheckResult.FAILED

        check = AnyFailingCheck()
        check.severity = Severities[BcSeverities.LOW]
        checks_allowlist = ['MEDIUM']
        report = runner.run(
            root_folder=None,
            files=[valid_plan_path],
            external_checks_dir=None,
            runner_filter=RunnerFilter(framework=["terraform_plan"], checks=checks_allowlist),
        )

        all_checks = report.failed_checks + report.passed_checks
        self.assertFalse(any(c.check_id == custom_check_id for c in all_checks))

    def test_runner_check_severity_filter_include(self):
        current_dir = os.path.dirname(os.path.realpath(__file__))
        valid_plan_path = current_dir + "/resources/plan/tfplan.json"
        runner = Runner()

        custom_check_id = "MY_CUSTOM_CHECK"

        resource_registry.checks = defaultdict(list)

        class AnyFailingCheck(BaseResourceCheck):
            def __init__(self, *_, **__) -> None:
                super().__init__(
                    "this should fail",
                    custom_check_id,
                    [CheckCategories.ENCRYPTION],
                    ["aws_db_instance"]
                )

            def scan_resource_conf(self, conf: Dict[str, Any]) -> CheckResult:
                return CheckResult.FAILED

        check = AnyFailingCheck()
        check.severity = Severities[BcSeverities.HIGH]
        checks_allowlist = ['MEDIUM']
        report = runner.run(
            root_folder=None,
            files=[valid_plan_path],
            external_checks_dir=None,
            runner_filter=RunnerFilter(framework=["terraform_plan"], checks=checks_allowlist),
        )

        all_checks = report.failed_checks + report.passed_checks
        self.assertTrue(any(c.check_id == custom_check_id for c in all_checks))

    def test_runner_check_skip_filter_omit(self):
        current_dir = os.path.dirname(os.path.realpath(__file__))
        valid_plan_path = current_dir + "/resources/plan/tfplan.json"
        runner = Runner()

        custom_check_id = "MY_CUSTOM_CHECK"

        resource_registry.checks = defaultdict(list)

        class AnyFailingCheck(BaseResourceCheck):
            def __init__(self, *_, **__) -> None:
                super().__init__(
                    "this should fail",
                    custom_check_id,
                    [CheckCategories.ENCRYPTION],
                    ["aws_db_instance"]
                )

            def scan_resource_conf(self, conf: Dict[str, Any]) -> CheckResult:
                return CheckResult.FAILED

        check = AnyFailingCheck()
        check.severity = Severities[BcSeverities.LOW]
        checks_denylist = ['MEDIUM']
        report = runner.run(
            root_folder=None,
            files=[valid_plan_path],
            external_checks_dir=None,
            runner_filter=RunnerFilter(framework=["terraform_plan"], skip_checks=checks_denylist),
        )

        all_checks = report.failed_checks + report.passed_checks
        self.assertFalse(any(c.check_id == custom_check_id for c in all_checks))

    def test_runner_check_skip_filter(self):
        current_dir = os.path.dirname(os.path.realpath(__file__))
        valid_plan_path = current_dir + "/resources/plan/tfplan.json"
        runner = Runner()

        custom_check_id = "MY_CUSTOM_CHECK"

        resource_registry.checks = defaultdict(list)

        class AnyFailingCheck(BaseResourceCheck):
            def __init__(self, *_, **__) -> None:
                super().__init__(
                    "this should fail",
                    custom_check_id,
                    [CheckCategories.ENCRYPTION],
                    ["aws_db_instance"]
                )

            def scan_resource_conf(self, conf: Dict[str, Any]) -> CheckResult:
                return CheckResult.FAILED

        check = AnyFailingCheck()
        check.severity = Severities[BcSeverities.HIGH]
        checks_denylist = ['MEDIUM']
        report = runner.run(
            root_folder=None,
            files=[valid_plan_path],
            external_checks_dir=None,
            runner_filter=RunnerFilter(framework=["terraform_plan"], skip_checks=checks_denylist),
        )

        all_checks = report.failed_checks + report.passed_checks
        self.assertTrue(any(c.check_id == custom_check_id for c in all_checks))

    def test_runner_child_modules(self):
        current_dir = os.path.dirname(os.path.realpath(__file__))
        valid_plan_path = current_dir + "/resources/plan_with_child_modules/tfplan.json"
        runner = Runner()
        runner.graph_registry.checks = []
        report = runner.run(
            root_folder=None,
            files=[valid_plan_path],
            external_checks_dir=None,
            runner_filter=RunnerFilter(framework=["all"]),
        )
        report_json = report.get_json()
        self.assertIsInstance(report_json, str)
        self.assertIsNotNone(report_json)
        self.assertIsNotNone(report.get_test_suite())
        self.assertEqual(report.get_exit_code({'soft_fail': False, 'soft_fail_checks': [], 'soft_fail_threshold': None, 'hard_fail_checks': [], 'hard_fail_threshold': None}), 1)
        self.assertEqual(report.get_exit_code({'soft_fail': True, 'soft_fail_checks': [], 'soft_fail_threshold': None, 'hard_fail_checks': [], 'hard_fail_threshold': None}), 0)

        self.assertEqual(report.get_summary()["failed"], 3)
        self.assertEqual(report.get_summary()["passed"], 4)

    def test_runner_nested_child_modules(self):
        current_dir = os.path.dirname(os.path.realpath(__file__))
        valid_plan_path = current_dir + "/resources/plan_nested_child_modules/tfplan.json"
        runner = Runner()
        runner.graph_registry.checks = []
        report = runner.run(
            root_folder=None,
            files=[valid_plan_path],
            external_checks_dir=[current_dir + "/extra_yaml_checks"],
            runner_filter=RunnerFilter(framework=["all"]),
        )
        report_json = report.get_json()
        self.assertIsInstance(report_json, str)
        self.assertIsNotNone(report_json)
        self.assertIsNotNone(report.get_test_suite())
        self.assertEqual(report.get_exit_code({'soft_fail': False, 'soft_fail_checks': [], 'soft_fail_threshold': None, 'hard_fail_checks': [], 'hard_fail_threshold': None}), 1)
        self.assertEqual(report.get_exit_code({'soft_fail': True, 'soft_fail_checks': [], 'soft_fail_threshold': None, 'hard_fail_checks': [], 'hard_fail_threshold': None}), 0)

        self.assertEqual(report.get_summary()["failed"], 15)
        self.assertEqual(report.get_summary()["passed"], 0)

        failed_check_ids = set([c.check_id for c in report.failed_checks])
        expected_failed_check_ids = {
            "CKV_AWS_37",
            "CKV_AWS_38",
            "CKV_AWS_39",
            "CKV_AWS_58",
            "CUSTOM_GRAPH_AWS_1"
        }

        assert failed_check_ids == expected_failed_check_ids

        # reset graph checks
        runner.graph_registry.checks = []
        runner.graph_registry.load_checks()

    def test_runner_root_module_resources_no_values(self):
        current_dir = os.path.dirname(os.path.realpath(__file__))
        valid_plan_path = current_dir + "/resources/plan_root_module_resources_no_values/tfplan.json"
        runner = Runner()
        runner.graph_registry.checks = []
        report = runner.run(
            root_folder=None,
            files=[valid_plan_path],
            external_checks_dir=None,
            runner_filter=RunnerFilter(framework=["all"]),
        )
        report_json = report.get_json()
        self.assertIsInstance(report_json, str)
        self.assertIsNotNone(report_json)
        self.assertIsNotNone(report.get_test_suite())
        self.assertEqual(report.get_exit_code({'soft_fail': False, 'soft_fail_checks': [], 'soft_fail_threshold': None, 'hard_fail_checks': [], 'hard_fail_threshold': None}), 1)
        self.assertEqual(report.get_exit_code({'soft_fail': True, 'soft_fail_checks': [], 'soft_fail_threshold': None, 'hard_fail_checks': [], 'hard_fail_threshold': None}), 0)

        # 4 checks fail on test data for single eks resource as of present
        # If more eks checks are added then this number will need to increase correspondingly to reflect
        # This reasoning holds for all current pass/fails in these tests
        self.assertEqual(report.get_summary()["failed"], 4)
        self.assertEqual(report.get_summary()["passed"], 0)

        failed_check_ids = set([c.check_id for c in report.failed_checks])
        expected_failed_check_ids = {
            "CKV_AWS_37",
            "CKV_AWS_38",
            "CKV_AWS_39",
            "CKV_AWS_58",
        }

        assert failed_check_ids == expected_failed_check_ids

        # reset graph checks
        runner.graph_registry.checks = []
        runner.graph_registry.load_checks()

    def test_runner_root_module_resources_no_values_route53(self):
        #given
        plan_file = Path(__file__).parent / "resources/plan_root_module_resources_no_values/tfplan_route53.json"

        # when
        report = Runner().run(
            root_folder=None,
            files=[str(plan_file)],
            external_checks_dir=None,
            runner_filter=RunnerFilter(framework=["terraform_plan"], checks=["CKV2_AWS_38", "CKV2_AWS_39"]),
        )

        # then
        summary = report.get_summary()

        self.assertEqual(summary["failed"], 1)
        self.assertEqual(summary["passed"], 1)

        passed_check_ids = set(c.check_id for c in report.passed_checks)
        expected_passed_check_ids = {"CKV2_AWS_39"}

        self.assertCountEqual(passed_check_ids, expected_passed_check_ids)

    def test_runner_data_resource_partial_values(self):
        # In rare circumstances a data resource with partial values in the plan could cause false negatives
        # Often 'data' does not even appear in the *_modules[x].resouces field within planned_values and is not scanned as expected
        # It can occur when tf module B depends on tf module A
        # And tf module A creates a resource that is used in a data block in tf module B
        # So some values can be known but other are not at plan time
        # This can cause the data block resource to be scanned as if it were a managed resource which is not configured correctly
        # See 'Modes': https://www.terraform.io/docs/internals/json-format.html#values-representation
        # This test verifies that such a circumstance stops occurring
        # There is a EKS Managed Resource and a EKS Data Resource
        # The EKS Managed Resource should have 4 failures corresponding with EKS checks.
        # The EKS Data Resource should not be scanned. Previously this would cause 8 failures.
        current_dir = os.path.dirname(os.path.realpath(__file__))
        valid_plan_path = current_dir + "/resources/plan_data_resource_partial_values/tfplan.json"
        runner = Runner()
        report = runner.run(
            root_folder=None,
            files=[valid_plan_path],
            external_checks_dir=None,
            runner_filter=RunnerFilter(framework=["all"]),
        )
        report_json = report.get_json()
        self.assertIsInstance(report_json, str)
        self.assertIsNotNone(report_json)
        self.assertIsNotNone(report.get_test_suite())
        self.assertEqual(report.get_exit_code({'soft_fail': False, 'soft_fail_checks': [], 'soft_fail_threshold': None, 'hard_fail_checks': [], 'hard_fail_threshold': None}), 1)
        self.assertEqual(report.get_exit_code({'soft_fail': True, 'soft_fail_checks': [], 'soft_fail_threshold': None, 'hard_fail_checks': [], 'hard_fail_threshold': None}), 0)

        self.assertEqual(report.get_summary()["failed"], 4)
        self.assertEqual(report.get_summary()["passed"], 0)

        failed_check_ids = set([c.check_id for c in report.failed_checks])
        expected_failed_check_ids = {
            "CKV_AWS_37",
            "CKV_AWS_38",
            "CKV_AWS_39",
            "CKV_AWS_58",
        }

        assert failed_check_ids == expected_failed_check_ids

    def test_runner_root_dir(self):
        current_dir = os.path.dirname(os.path.realpath(__file__))
        root_dir = current_dir + "/resources"
        runner = Runner()
        report = runner.run(
            root_folder=root_dir, files=None, external_checks_dir=None, runner_filter=RunnerFilter(framework=["all"])
        )
        report_json = report.get_json()
        self.assertIsInstance(report_json, str)
        self.assertIsNotNone(report_json)
        self.assertIsNotNone(report.get_test_suite())
        self.assertEqual(report.get_exit_code({'soft_fail': False, 'soft_fail_checks': [], 'soft_fail_threshold': None, 'hard_fail_checks': [], 'hard_fail_threshold': None}), 1)
        self.assertEqual(report.get_exit_code({'soft_fail': True, 'soft_fail_checks': [], 'soft_fail_threshold': None, 'hard_fail_checks': [], 'hard_fail_threshold': None}), 0)

        self.assertGreaterEqual(report.get_summary()["failed"], 71)
        self.assertGreaterEqual(report.get_summary()["passed"], 65)

        files_scanned = list(set(map(lambda rec: rec.file_path, report.failed_checks)))
        self.assertGreaterEqual(len(files_scanned), 6)

    def test_runner_honors_enforcement_rules(self):
        current_dir = os.path.dirname(os.path.realpath(__file__))
        root_dir = current_dir + "/resources"
        runner = Runner()
        filter = RunnerFilter(framework=['terraform_plan'], use_enforcement_rules=True)
        # this is not quite a true test, because the checks don't have severities. However, this shows that the check registry
        # passes the report type properly to RunnerFilter.should_run_check, and we have tests for that method
        filter.enforcement_rule_configs = {CheckType.TERRAFORM_PLAN: Severities[BcSeverities.OFF]}
        report = runner.run(
            root_folder=root_dir, files=None, external_checks_dir=None, runner_filter=filter
        )
        self.assertEqual(len(report.failed_checks), 0)
        self.assertEqual(len(report.passed_checks), 0)
        self.assertEqual(len(report.skipped_checks), 0)

    def test_record_relative_path_with_relative_dir(self):
        # this is just constructing the scan dir as normal
        current_dir = os.path.dirname(os.path.realpath(__file__))
        scan_dir_path = os.path.join(current_dir, "resources", "plan")

        # this is the relative path to the directory to scan (what would actually get passed to the -d arg)
        dir_rel_path = os.path.relpath(scan_dir_path).replace('\\', '/')

        runner = Runner()
        checks_allowlist = ["CKV_AWS_6"]
        report = runner.run(
            root_folder=dir_rel_path,
            external_checks_dir=None,
            runner_filter=RunnerFilter(framework=["terraform"], checks=checks_allowlist),
        )

        all_checks = report.failed_checks + report.passed_checks
        for record in all_checks:
            self.assertEqual(record.repo_file_path, f'/{os.path.join(dir_rel_path, record.file_path.lstrip("/"))}')

    def test_record_relative_path_with_relative_file(self):
        # this is just constructing the scan dir as normal
        current_dir = os.path.dirname(os.path.realpath(__file__))
        scan_file_path = os.path.join(current_dir, "resources", "plan", "tfplan.json")

        # this is the relative path to the file to scan (what would actually get passed to the -f arg)
        file_rel_path = os.path.relpath(scan_file_path)

        runner = Runner()
        checks_allowlist = ["CKV_AWS_20"]
        report = runner.run(
            root_folder=None,
            external_checks_dir=None,
            files=[file_rel_path],
            runner_filter=RunnerFilter(framework=["terraform"], checks=checks_allowlist),
        )

        all_checks = report.failed_checks + report.passed_checks
        for record in all_checks:
            self.assertEqual(record.repo_file_path, f'/{file_rel_path}')

    def test_runner_unexpected_eks_node_group_remote_access(self):
        current_dir = os.path.dirname(os.path.realpath(__file__))
        valid_plan_path = current_dir + "/resources/unexpected/eks_node_group_remote_access.json"
        runner = Runner()
        report = runner.run(
            root_folder=None,
            files=[valid_plan_path],
            external_checks_dir=None,
            runner_filter=RunnerFilter(framework=["all"]),
        )
        report_json = report.get_json()
        self.assertIsInstance(report_json, str)
        self.assertIsNotNone(report_json)
        self.assertIsNotNone(report.get_test_suite())
        self.assertEqual(report.get_exit_code({'soft_fail': False, 'soft_fail_checks': [], 'soft_fail_threshold': None, 'hard_fail_checks': [], 'hard_fail_threshold': None}), 0)
        self.assertEqual(report.get_exit_code({'soft_fail': True, 'soft_fail_checks': [], 'soft_fail_threshold': None, 'hard_fail_checks': [], 'hard_fail_threshold': None}), 0)

        self.assertEqual(report.get_summary()["failed"], 0)
        self.assertEqual(report.get_summary()["passed"], 1)

    def test_runner_with_resource_reference(self):
        current_dir = os.path.dirname(os.path.realpath(__file__))
        valid_plan_path = current_dir + "/resources/plan_with_resource_reference/tfplan.json"
        allowed_checks = ["CKV_AWS_84"]

        report = Runner().run(
            root_folder=None,
            files=[valid_plan_path],
            external_checks_dir=None,
            runner_filter=RunnerFilter(framework=["all"], checks=allowed_checks),
        )

        report_json = report.get_json()
        self.assertIsInstance(report_json, str)
        self.assertIsNotNone(report_json)
        self.assertIsNotNone(report.get_test_suite())
        self.assertEqual(report.get_exit_code({'soft_fail': False, 'soft_fail_checks': [], 'soft_fail_threshold': None, 'hard_fail_checks': [], 'hard_fail_threshold': None}), 0)
        self.assertEqual(report.get_exit_code({'soft_fail': True, 'soft_fail_checks': [], 'soft_fail_threshold': None, 'hard_fail_checks': [], 'hard_fail_threshold': None}), 0)

        self.assertEqual(report.get_summary()["failed"], 0)
        self.assertEqual(report.get_summary()["passed"], 1)

    def test_runner_with_resource_reference_graph_check(self):
        # given
        valid_plan_path = Path(__file__).parent / "resources/plan_with_resource_reference/tfplan_graph.json"
        allowed_checks = ["CKV2_AWS_6"]

        # when
        report = Runner().run(
            root_folder=None,
            files=[str(valid_plan_path)],
            external_checks_dir=None,
            runner_filter=RunnerFilter(framework=["terraform_plan"], checks=allowed_checks),
        )

        # then
        summary = report.get_summary()

        self.assertEqual(summary["failed"], 0)
        self.assertEqual(summary["passed"], 1)

    def test_runner_with_resource_reference_extra_ref(self):
        # given
        valid_plan_path = Path(__file__).parent / "resources/plan_with_resource_reference/tfplan_extra_ref.json"
        extra_checks_dir_path = [str(Path(__file__).parent / "extra_tf_plan_checks")]
        allowed_checks = ["CUSTOM_CONNECTION_1"]

        # when
        report = Runner().run(
            root_folder=None,
            files=[str(valid_plan_path)],
            external_checks_dir=extra_checks_dir_path,
            runner_filter=RunnerFilter(framework=["terraform_plan"], checks=allowed_checks),
        )

        # then
        summary = report.get_summary()

        self.assertEqual(summary["failed"], 1)
        self.assertEqual(summary["passed"], 1)
        self.assertEqual(summary["skipped"], 0)
        self.assertEqual(summary["parsing_errors"], 0)
        self.assertEqual(summary["resource_count"], 4)

    def test_runner_skip_graph_when_no_plan_exists(self):
        # given
        tf_file_path = Path(__file__).parent / "resource/example/example.tf"

        # when
        report = Runner().run(
            root_folder=None,
            files=[str(tf_file_path)],
            external_checks_dir=None,
            runner_filter=RunnerFilter(framework=["terraform_plan"]),
        )

        # then
        summary = report.get_summary()

        self.assertEqual(summary["failed"], 0)
        self.assertEqual(summary["passed"], 0)
        self.assertEqual(summary["skipped"], 0)
        self.assertEqual(summary["parsing_errors"], 0)
        self.assertEqual(summary["resource_count"], 0)

    def test_runner_utf_16_encoded(self):
        # given
        tf_file_path = Path(__file__).parent / "resources/plan_with_utf_16_encoding/tfplan.json"

        # when
        report = Runner().run(
            root_folder=None,
            files=[str(tf_file_path)],
            external_checks_dir=None,
            runner_filter=RunnerFilter(framework=["terraform_plan"]),
        )

        # then
        summary = report.get_summary()

        self.assertGreater(summary["failed"], 0)
        self.assertGreater(summary["passed"], 0)
        self.assertEqual(summary["skipped"], 0)
        self.assertEqual(summary["parsing_errors"], 0)

    def test_runner_line_numbers(self):
        # given
        tf_file_path = Path(__file__).parent / "resources/plan_with_resource_reference/tfplan.json"

        # when
        report = Runner().run(
            root_folder=None,
            files=[str(tf_file_path)],
            external_checks_dir=None,
            runner_filter=RunnerFilter(framework=["terraform_plan"]),
        )

        # then
        failed_check = report.failed_checks[0]
        self.assertEqual(failed_check.file_line_range, [13, 19])

    def test_runner_ignore_lifecycle_checks(self):
        # given
        tf_file_path = Path(__file__).parent / "resources/plan_with_lifecycle_check/tfplan.json"

        # when
        report = Runner().run(
            root_folder=None,
            files=[str(tf_file_path)],
            external_checks_dir=None,
            runner_filter=RunnerFilter(framework=["terraform_plan"]),
        )

        # then
        self.assertEqual(len(report.failed_checks), 0)

    def test_runner_extra_check(self):
        # given
        current_dir = Path(__file__).parent
        tf_dir_path = str(current_dir / "resources/plan_with_deleted_resources")
        extra_checks_dir_path = [str(current_dir / "extra_tf_plan_checks")]

        # when
        report = Runner().run(
            root_folder=tf_dir_path,
            external_checks_dir=extra_checks_dir_path,
            runner_filter=RunnerFilter(checks=["CUSTOM_DELETE_1", "CUSTOM_DELETE_2"])
        )

        # then
        summary = report.get_summary()
        self.assertEqual(summary["failed"], 2)

        resource_ids = [check.resource for check in report.failed_checks]
        self.assertCountEqual(resource_ids,["aws_secretsmanager_secret.default", "aws_secretsmanager_secret.default"])

        # check also the details
        failed_check = next(check for check in report.failed_checks if check.check_id == "CUSTOM_DELETE_1")
        self.assertEqual(failed_check.details, ["some great details"])

    def test_runner_nested_child_modules_with_connections(self):
        # given
        tf_file_path = Path(__file__).parent / "resources/plan_nested_child_modules_with_connections/tfplan.json"

        passing_resources = {
            "module.s3_module.module.s3_submodule.aws_s3_bucket.submodule_bucket",
            "module.s3_module.aws_s3_bucket.module_bucket",
            "aws_s3_bucket.root_bucket",
        }
        failing_resources = {
            "module.s3_bucket.aws_s3_bucket.this[0]",
        }

        # when
        report = Runner().run(
            root_folder=None,
            files=[str(tf_file_path)],
            external_checks_dir=None,
            runner_filter=RunnerFilter(framework=["terraform_plan"], checks=["CKV2_AWS_6"]),
        )

        # then
        self.assertEqual(len(report.passed_checks), 3)
        self.assertEqual(len(report.failed_checks), 1)

        passed_check_resources = {c.resource for c in report.passed_checks}
        failed_check_resources = {c.resource for c in report.failed_checks}

        self.assertEqual(passing_resources, passed_check_resources)
        self.assertEqual(failing_resources, failed_check_resources)

    def test_runner_with_iam_policies(self):
        # given
        tf_file_path = Path(__file__).parent / "resources/plan_with_iam_policies/tfplan.json"

        passing_resources = {
            "aws_iam_policy.policy_pass",
        }
        failing_resources = {
            "aws_iam_role_policy.fail_1",
            "aws_iam_group_policy.fail_2",
            "aws_iam_user_policy.fail_3",
        }

        # when
        report = Runner().run(
            root_folder=None,
            files=[str(tf_file_path)],
            external_checks_dir=None,
            runner_filter=RunnerFilter(framework=["terraform_plan"], checks=["CKV2_AWS_40", "CKV_AWS_287"]),
        )

        # then
        summary = report.get_summary()

        self.assertEqual(summary["passed"], 2)  # "aws_iam_policy.policy_pass" passes both checks
        self.assertEqual(summary["failed"], 3)

        passed_check_resources = {c.resource for c in report.passed_checks}
        failed_check_resources = {c.resource for c in report.failed_checks}

        self.assertEqual(passing_resources, passed_check_resources)
        self.assertEqual(failing_resources, failed_check_resources)

    def test_runner_with_iam_data_block(self):
        # given
        tf_file_path = Path(__file__).parent / "resources/plan_with_iam_data_block/tfplan.json"

        failing_resources = {
            "data.aws_iam_policy_document.allow_access_from_another_account",
        }

        # when
        report = Runner().run(
            root_folder=None,
            files=[str(tf_file_path)],
            external_checks_dir=None,
            runner_filter=RunnerFilter(framework=["terraform_plan"], checks=["CKV_AWS_49"]),
        )

        # then
        summary = report.get_summary()
        self.assertEqual(summary["passed"], 0)
        self.assertEqual(summary["failed"], 1)

        failed_check_resources = {c.resource for c in report.failed_checks}
        self.assertEqual(failing_resources, failed_check_resources)

    @mock.patch.dict(os.environ, {'CHECKOV_ENABLE_NESTED_MODULES': 'True', 'CHECKOV_EXPERIMENTAL_CROSS_VARIABLE_EDGES': 'True'})
    def test_plan_and_tf_combine_graph(self):
        tf_file_path = Path(__file__).parent / "resources/plan_and_tf_combine_graph/tfplan.json"

        repo_path = Path(__file__).parent / "resources/plan_and_tf_combine_graph"

        # deep_analysis disabled
        report = Runner().run(
            root_folder=None,
            files=[str(tf_file_path)],
            external_checks_dir=None,
            runner_filter=RunnerFilter(framework=["terraform_plan"], checks=["CKV2_AWS_6"], deep_analysis=False,
                                       repo_root_for_plan_enrichment=[repo_path])
        )

        self.assertEqual(len(report.passed_checks), 0)
        self.assertEqual(len(report.failed_checks), 2)

        # deep_analysis enabled
        report = Runner().run(
            root_folder=None,
            files=[str(tf_file_path)],
            external_checks_dir=None,
            runner_filter=RunnerFilter(framework=["terraform_plan"], checks=["CKV2_AWS_6"], deep_analysis=True, repo_root_for_plan_enrichment=[repo_path])
        )

        self.assertEqual(len(report.passed_checks), 2)
        self.assertEqual(len(report.failed_checks), 0)

        expected_addresses = ['aws_s3_bucket.example', 'aws_s3_bucket.example_2']
        report_addresses = [report.passed_checks[0].resource_address, report.passed_checks[1].resource_address]
        assert sorted(expected_addresses) == sorted(report_addresses)
        assert report.passed_checks[0].file_path.endswith('.json')
        assert report.passed_checks[1].file_path.endswith('.json')

    @mock.patch.dict(os.environ, {'CHECKOV_ENABLE_NESTED_MODULES': 'False'})
    def test_plan_resources_ids(self):
        current_dir = os.path.dirname(os.path.realpath(__file__))
        valid_plan_path = current_dir + "/resources/plan_resources_ids/tfplan.json"
        valid_resources_ids = ["module.child_1_c.aws_eks_cluster.cluster", "module.child_1_b.aws_eks_cluster.cluster",
                               "module.child_1_a.aws_eks_cluster.cluster"]
        runner = Runner()
        runner.graph_registry.checks = []
        report = runner.run(
            root_folder=None,
            files=[valid_plan_path],
            external_checks_dir=[current_dir + "/extra_yaml_checks"],
            runner_filter=RunnerFilter(framework=["terraform_plan"]),
        )
        self.assertGreater(report.get_summary()["failed"] + report.get_summary()["passed"], 0)

        for check in itertools.chain(report.failed_checks, report.passed_checks):
            self.assertIn(check.resource, valid_resources_ids)

        self.assertEqual(len(report.resources), 3)

    @mock.patch.dict(os.environ, {'CHECKOV_ENABLE_NESTED_MODULES': 'True'})
    def test_plan_resources_ids_with_nested_modules(self):
        current_dir = os.path.dirname(os.path.realpath(__file__))
        valid_plan_path = current_dir + "/resources/plan_resources_ids_with_nested_modules/tfplan.json"
        valid_resources_ids = ["module.child_0.module.child_1_c.aws_eks_cluster.cluster",
                               "module.child_0.module.child_1_b.aws_eks_cluster.cluster",
                               "module.child_0.module.child_1_a.aws_eks_cluster.cluster"]
        runner = Runner()
        runner.graph_registry.checks = []
        report = runner.run(
            root_folder=None,
            files=[valid_plan_path],
            external_checks_dir=[current_dir + "/extra_yaml_checks"],
            runner_filter=RunnerFilter(framework=["terraform_plan"]),
        )
        self.assertGreater(report.get_summary()["failed"] + report.get_summary()["passed"], 0)

        for check in itertools.chain(report.failed_checks, report.passed_checks):
            self.assertIn(check.resource, valid_resources_ids)

        self.assertEqual(len(report.resources), 3)
        
    @mock.patch.dict(os.environ, {'CHECKOV_ENABLE_NESTED_MODULES': 'True'})
    def test_plan_resources_created_by_modules(self):
        current_dir = os.path.dirname(os.path.realpath(__file__))
        valid_plan_path = current_dir + "/extra_tf_plan_checks/modules.json"
        runner = Runner()
        report = runner.run(
            root_folder=None, external_checks_dir=None, files=[valid_plan_path],
            runner_filter=RunnerFilter(checks=['CKV2_GCP_12', 'CKV_GCP_88'])
        )
        passed_checks_CKV2_GCP_12 = [check for check in report.passed_checks if check.check_id == 'CKV2_GCP_12']
        passed_checks_CKV_GCP_88 = [check for check in report.passed_checks if check.check_id == 'CKV_GCP_88']
        
        assert passed_checks_CKV2_GCP_12[0].resource == 'module.achia_test_valid_443.google_compute_firewall.custom[0]'
        assert passed_checks_CKV2_GCP_12[1].resource == 'module.achia_test_valid_ports.google_compute_firewall.custom[0]'
        assert passed_checks_CKV2_GCP_12[2].resource == 'module.achia_test_violating_no_ports.google_compute_firewall.custom[0]'
        assert passed_checks_CKV2_GCP_12[3].resource == 'module.achia_test_violating_port.google_compute_firewall.custom[0]'
        
        assert passed_checks_CKV_GCP_88[0].resource == 'module.achia_test_valid_443.google_compute_firewall.custom[0]'
        assert passed_checks_CKV_GCP_88[1].resource == 'module.achia_test_valid_ports.google_compute_firewall.custom[0]'
        assert passed_checks_CKV_GCP_88[2].resource == 'module.achia_test_violating_no_ports.google_compute_firewall.custom[0]'
        assert passed_checks_CKV_GCP_88[3].resource == 'module.achia_test_violating_port.google_compute_firewall.custom[0]'

    def test___get_file_path__with_tf_definition_key_uses_correct_file_path(self):
        tf_definition = TFDefinitionKey(file_path='test')
        file_path, scanned_file = Runner()._get_file_path(tf_definition, 'test')
        assert file_path == 'test'
        assert scanned_file == '/.'

    def tearDown(self) -> None:
        resource_registry.checks = self.orig_checks


if __name__ == "__main__":
    unittest.main()



from typing import Any

from checkov.common.models.enums import CheckCategories
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck


class CosmosDBDisableAccessKeyWrite(BaseResourceValueCheck):
    def __init__(self) -> None:
        name = "Ensure cosmosdb does not allow privileged escalation by restricting management plane changes"
        id = "CKV_AZURE_132"
        supported_resources = ("azurerm_cosmosdb_account",)
        categories = (CheckCategories.GENERAL_SECURITY,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self) -> str:
        return "access_key_metadata_writes_enabled"

    def get_expected_value(self) -> Any:
        return False


check = CosmosDBDisableAccessKeyWrite()



import itertools
import os
import unittest
from pathlib import Path

import pytest

from checkov.cloudformation.runner import Runner
from checkov.runner_filter import RunnerFilter


class TestRunningGraphChecks(unittest.TestCase):

    @pytest.mark.skip("Graph checks have not been written yet for cloudformation")
    def test_runner(self):
        dir_path = os.path.join(os.path.dirname(os.path.realpath(__file__)), "resources")
        report = Runner().run(dir_path)
        assert any(
            check.check_id == "CKV2_AWS_24" for check in itertools.chain(report.failed_checks, report.passed_checks))
        assert any(
            check.check_id == "CKV2_AWS_25" for check in itertools.chain(report.failed_checks, report.passed_checks))
        assert any(
            check.check_id == "CKV2_AWS_26" for check in itertools.chain(report.failed_checks, report.passed_checks))

    @pytest.mark.skip("Graph checks have not been written yet for cloudformation")
    def test_runner_sam(self):
        # given
        test_dir_path = Path(__file__).parent.parent / "graph_builder/resources/sam"

        # when
        report = Runner().run(root_folder=str(test_dir_path), runner_filter=RunnerFilter(checks=["CKV2_AWS_26"]))

        # then
        summary = report.get_summary()

        passing_resources = {
            "AWS::Serverless::Function.Function1",
            "AWS::Serverless::Function.Function2",
        }

        passed_check_resources = {c.resource for c in report.passed_checks}

        self.assertEqual(summary["passed"], 2)
        self.assertEqual(summary["failed"], 0)
        self.assertEqual(summary["skipped"], 0)
        self.assertEqual(summary["parsing_errors"], 0)

        self.assertEqual(passing_resources, passed_check_resources)


if __name__ == '__main__':
    unittest.main()



import os
from parameterized import parameterized_class

from tests.terraform.graph.checks_infra.test_base import TestBaseSolver

TEST_DIRNAME = os.path.dirname(os.path.realpath(__file__))


@parameterized_class([
   {"graph_framework": "NETWORKX"},
   {"graph_framework": "IGRAPH"}
])
class TestIntersectsSolver(TestBaseSolver):
    def setUp(self):
        self.checks_dir = TEST_DIRNAME
        super(TestIntersectsSolver, self).setUp()

    def test_simple_array_intersection1(self):
        root_folder = '../../../resources/public_virtual_machines'
        check_id = "PublicVMs"
        should_pass = ['aws_default_security_group.default_security_group_open']
        should_fail = ['aws_default_security_group.default_security_group_closed']
        expected_results = {check_id: {"should_pass": should_pass, "should_fail": should_fail}}

        self.run_test(root_folder=root_folder, expected_results=expected_results, check_id=check_id)

    def test_simple_array_intersection2(self):
        root_folder = '../../../resources/array_test'
        check_id = "ArrayIntersect"
        should_pass = ['aws_xyz.pass1', 'aws_xyz.pass2']
        should_fail = ['aws_xyz.fail2', 'aws_xyz.fail3', 'aws_xyz.pass3']
        expected_results = {check_id: {"should_pass": should_pass, "should_fail": should_fail}}

        self.run_test(root_folder=root_folder, expected_results=expected_results, check_id=check_id)

    def test_none_attribute(self):
        root_folder = '../../../resources/public_virtual_machines'
        check_id = "NoneAttribute"
        should_pass = []
        should_fail = ['aws_subnet.subnet_public_ip', 'aws_subnet.subnet_not_public_ip']
        expected_results = {check_id: {"should_pass": should_pass, "should_fail": should_fail}}

        self.run_test(root_folder=root_folder, expected_results=expected_results, check_id=check_id)

    def test_string_attribute(self):
        root_folder = '../../../resources/public_virtual_machines'
        check_id = "StringAttribute"
        should_pass = ['aws_subnet.subnet_public_ip']
        should_fail = ['aws_subnet.subnet_not_public_ip']
        expected_results = {check_id: {"should_pass": should_pass, "should_fail": should_fail}}

        self.run_test(root_folder=root_folder, expected_results=expected_results, check_id=check_id)

    def test_mixed_value(self):
        root_folder = '../../../resources/public_virtual_machines'
        check_id = "MixedValue"
        should_pass = ['aws_default_security_group.default_security_group_open']
        should_fail = ['aws_default_security_group.default_security_group_closed']
        expected_results = {check_id: {"should_pass": should_pass, "should_fail": should_fail}}

        self.run_test(root_folder=root_folder, expected_results=expected_results, check_id=check_id)

    def test_tags_intersection(self):
        root_folder = '../../../resources/tag_includes'
        check_id = "TagsIntersect"
        should_pass = ['aws_subnet.acme_subnet']
        should_fail = ['aws_instance.some_instance', 'aws_s3_bucket.acme_s3_bucket']
        expected_results = {check_id: {"should_pass": should_pass, "should_fail": should_fail}}

        self.run_test(root_folder=root_folder, expected_results=expected_results, check_id=check_id)



import unittest
from pathlib import Path

from checkov.cloudformation.checks.resource.aws.ECRPolicy import check
from checkov.cloudformation.runner import Runner
from checkov.runner_filter import RunnerFilter


class TestECRPolicy(unittest.TestCase):
    def test_summary(self):
        # given
        test_files_dir = Path(__file__).parent / "example_ECRPolicy"

        # when
        report = Runner().run(root_folder=str(test_files_dir), runner_filter=RunnerFilter(checks=[check.id]))

        # then
        summary = report.get_summary()

        passing_resources = {
            "AWS::ECR::Repository.Restricted",
            "AWS::ECR::Repository.vpc16AA8B31E",
            "AWS::ECR::Repository.CondAllPass",
            "AWS::ECR::Repository.CondAnyPass",
            "AWS::ECR::Repository.CondEqualsPass",
        }
        failing_resources = {
            "AWS::ECR::Repository.Public",
        }

        passed_check_resources = {c.resource for c in report.passed_checks}
        failed_check_resources = {c.resource for c in report.failed_checks}

        self.assertEqual(summary['passed'], len(passing_resources))
        self.assertEqual(summary['failed'], len(failing_resources))
        self.assertEqual(summary['skipped'], 0)
        self.assertEqual(summary['parsing_errors'], 0)

        self.assertEqual(passing_resources, passed_check_resources)
        self.assertEqual(failing_resources, failed_check_resources)


if __name__ == '__main__':
    unittest.main()



import os
import shutil
import unittest
from pathlib import Path
from unittest import mock

import pytest

from checkov.common.util.consts import DEFAULT_EXTERNAL_MODULES_DIR
from checkov.common.util.parser_utils import TERRAFORM_NESTED_MODULE_PATH_PREFIX, TERRAFORM_NESTED_MODULE_PATH_ENDING, \
    TERRAFORM_NESTED_MODULE_INDEX_SEPARATOR
from checkov.terraform.parser import Parser


@pytest.fixture
def tmp_path(request, tmp_path: Path):
    # https://pytest.org/en/latest/how-to/unittest.html#mixing-pytest-fixtures-into-unittest-testcase-subclasses-using-marks
    request.cls.tmp_path = tmp_path


@pytest.mark.usefixtures("tmp_path")
class TestParserInternals(unittest.TestCase):

    def setUp(self) -> None:
        from checkov.terraform.module_loading.registry import ModuleLoaderRegistry

        # needs to be reset, because the cache belongs to the class not instance
        ModuleLoaderRegistry.module_content_cache = {}

        self.resources_dir = os.path.realpath(os.path.join(os.path.dirname(os.path.realpath(__file__)),
                                                           "./resources"))
        self.external_module_path = ''

    def tearDown(self) -> None:
        if os.path.exists(self.external_module_path):
            shutil.rmtree(self.external_module_path)

    def test_load_registry_module(self):
        parser = Parser()
        directory = os.path.join(self.resources_dir, "registry_security_group")
        self.external_module_path = os.path.join(directory, DEFAULT_EXTERNAL_MODULES_DIR)
        out_definitions = {}
        parser.parse_directory(directory=directory, out_definitions=out_definitions,
                               out_evaluations_context={},
                               download_external_modules=True,
                               external_modules_download_path=DEFAULT_EXTERNAL_MODULES_DIR)

        external_aws_modules_path = os.path.join(self.external_module_path, 'github.com/terraform-aws-modules/terraform-aws-security-group/v3.18.0')
        assert os.path.exists(external_aws_modules_path)

    @mock.patch.dict(os.environ, {"CHECKOV_ENABLE_NESTED_MODULES": "True"})
    def test_load_inner_registry_module_with_nested_modules(self):
        self.load_inner_registry_module(True)

    @mock.patch.dict(os.environ, {"CHECKOV_ENABLE_NESTED_MODULES": "False"})
    def test_load_inner_registry_module_without_nested_modules(self):
        self.load_inner_registry_module(False)

    def load_inner_registry_module(self, nested_modules):
        parser = Parser()
        directory = os.path.join(self.resources_dir, "registry_security_group_inner_module")
        self.external_module_path = os.path.join(self.tmp_path, DEFAULT_EXTERNAL_MODULES_DIR)
        out_definitions = {}
        parser.parse_directory(directory=directory, out_definitions=out_definitions,
                               out_evaluations_context={},
                               download_external_modules=True,
                               external_modules_download_path=self.external_module_path)
        self.assertEqual(11, len(list(out_definitions.keys())))
        expected_remote_module_path = f'{self.external_module_path}/github.com/terraform-aws-modules/terraform-aws-security-group/v4.0.0'
        expected_inner_remote_module_path = f'{expected_remote_module_path}/modules/http-80'
        expected_main_file = os.path.join(directory, 'main.tf')
        expected_inner_main_file = os.path.join(directory, expected_inner_remote_module_path, 'main.tf')
        expected_file_names = [
            expected_main_file,
            os.path.join(directory, expected_inner_remote_module_path, f'auto_values.tf{TERRAFORM_NESTED_MODULE_PATH_PREFIX}{expected_main_file}{TERRAFORM_NESTED_MODULE_INDEX_SEPARATOR}0{TERRAFORM_NESTED_MODULE_PATH_ENDING}'),
            os.path.join(directory, expected_inner_remote_module_path, f'main.tf{TERRAFORM_NESTED_MODULE_PATH_PREFIX}{expected_main_file}{TERRAFORM_NESTED_MODULE_INDEX_SEPARATOR}0{TERRAFORM_NESTED_MODULE_PATH_ENDING}'),
            os.path.join(directory, expected_inner_remote_module_path, f'outputs.tf{TERRAFORM_NESTED_MODULE_PATH_PREFIX}{expected_main_file}{TERRAFORM_NESTED_MODULE_INDEX_SEPARATOR}0{TERRAFORM_NESTED_MODULE_PATH_ENDING}'),
            os.path.join(directory, expected_inner_remote_module_path, f'variables.tf{TERRAFORM_NESTED_MODULE_PATH_PREFIX}{expected_main_file}{TERRAFORM_NESTED_MODULE_INDEX_SEPARATOR}0{TERRAFORM_NESTED_MODULE_PATH_ENDING}'),
            os.path.join(directory, expected_inner_remote_module_path, f'versions.tf{TERRAFORM_NESTED_MODULE_PATH_PREFIX}{expected_main_file}{TERRAFORM_NESTED_MODULE_INDEX_SEPARATOR}0{TERRAFORM_NESTED_MODULE_PATH_ENDING}'),

            os.path.join(directory, expected_remote_module_path, f'main.tf{TERRAFORM_NESTED_MODULE_PATH_PREFIX}{expected_inner_main_file}{TERRAFORM_NESTED_MODULE_INDEX_SEPARATOR}0{TERRAFORM_NESTED_MODULE_PATH_ENDING}'),
            os.path.join(directory, expected_remote_module_path, f'outputs.tf{TERRAFORM_NESTED_MODULE_PATH_PREFIX}{expected_inner_main_file}{TERRAFORM_NESTED_MODULE_INDEX_SEPARATOR}0{TERRAFORM_NESTED_MODULE_PATH_ENDING}'),
            os.path.join(directory, expected_remote_module_path, f'rules.tf{TERRAFORM_NESTED_MODULE_PATH_PREFIX}{expected_inner_main_file}{TERRAFORM_NESTED_MODULE_INDEX_SEPARATOR}0{TERRAFORM_NESTED_MODULE_PATH_ENDING}'),
            os.path.join(directory, expected_remote_module_path, f'variables.tf{TERRAFORM_NESTED_MODULE_PATH_PREFIX}{expected_inner_main_file}{TERRAFORM_NESTED_MODULE_INDEX_SEPARATOR}0{TERRAFORM_NESTED_MODULE_PATH_ENDING}'),
            os.path.join(directory, expected_remote_module_path, f'versions.tf{TERRAFORM_NESTED_MODULE_PATH_PREFIX}{expected_inner_main_file}{TERRAFORM_NESTED_MODULE_INDEX_SEPARATOR}0{TERRAFORM_NESTED_MODULE_PATH_ENDING}'),
        ]

        if not nested_modules:
            for expected_file_name in expected_file_names:
                if expected_file_name not in list(out_definitions.keys()):
                    self.fail(f"expected file {expected_file_name} to be in out_definitions")
        else:
            for expected_file_name in expected_file_names:
                if not any(definition for definition in out_definitions.keys() if definition.startswith(expected_file_name[:-3])):
                    self.fail(f"expected file {expected_file_name} to be in out_definitions")

    def test_invalid_module_sources(self):
        parser = Parser()
        directory = os.path.join(self.resources_dir, "failing_module_address")
        self.external_module_path = os.path.join(directory, DEFAULT_EXTERNAL_MODULES_DIR)
        out_definitions = {}
        parser.parse_directory(directory=directory, out_definitions=out_definitions,
                               out_evaluations_context={},
                               download_external_modules=True,
                               external_modules_download_path=DEFAULT_EXTERNAL_MODULES_DIR)
        # check that only the original file was parsed successfully without getting bad external modules
        self.assertEqual(1, len(list(out_definitions.keys())))

    def test_malformed_output_blocks(self):
        parser = Parser()
        directory = os.path.join(self.resources_dir, "malformed_outputs")
        self.external_module_path = os.path.join(directory, DEFAULT_EXTERNAL_MODULES_DIR)
        out_definitions = {}
        parser.parse_directory(directory=directory, out_definitions=out_definitions,
                               out_evaluations_context={},
                               download_external_modules=True,
                               external_modules_download_path=DEFAULT_EXTERNAL_MODULES_DIR)
        file_path, entity_definitions = next(iter(out_definitions.items()))
        self.assertEqual(2, len(list(out_definitions[file_path]['output'])))

    def test_load_local_module(self):
        # given
        parser = Parser()
        directory = os.path.join(self.resources_dir, "local_module")
        out_definitions = {}

        # when
        parser.parse_directory(
            directory=directory, out_definitions=out_definitions, out_evaluations_context={}
        )

        # then
        self.assertEqual(len(out_definitions), 3)  # root file + 2x module file
        self.assertEqual(len(parser.loaded_files_map), 2)  # root file + 1x module file

    def test_load_nested_dup_module(self):
        parser = Parser()
        directory = os.path.join(self.resources_dir, "parser_dup_nested")
        out_definitions = {}
        parser.parse_directory(directory=directory, out_evaluations_context={}, out_definitions=out_definitions)

        self.assertEqual(len(out_definitions), 7)
        self.assertEqual(len(parser.loaded_files_map), 3)

    def test_load_local_nested_module(self):
        # given
        parser = Parser()
        directory = os.path.join(self.resources_dir, "parser_nested_modules")
        out_definitions = {}

        # when
        parser.parse_directory(
            directory=directory, out_definitions=out_definitions, out_evaluations_context={}
        )

        # then
        self.assertEqual(len(out_definitions), 5)  # root file + 2x module file
        self.assertEqual(len(parser.loaded_files_map), 5)  # root file + 1x module file



from pathlib import Path

from checkov.common.output.report import CheckType

from checkov.common.bridgecrew.bc_source import get_source_type
from checkov.github_actions.runner import Runner

from checkov.runner_filter import RunnerFilter
from pytest_mock import MockerFixture

from tests.common.image_referencer.test_utils import mock_get_license_statuses_async, mock_get_image_cached_result_async

RESOURCES_PATH = Path(__file__).parent.parent / "resources/.github/workflows"


def test_github_action_workflow(mocker: MockerFixture):
    from checkov.common.bridgecrew.platform_integration import bc_integration

    file_name = "workflow_with_string_container.yml"
    image_name = "node:14.16"
    image_id = "sha256:2460522297"
    code_lines = "12-13"
    image_resource_postfixes = ["go", "openssl", "musl"]
    test_file = RESOURCES_PATH / file_name

    runner_filter = RunnerFilter(run_image_referencer=True)
    bc_integration.bc_source = get_source_type("disabled")

    mocker.patch(
        "checkov.common.images.image_referencer.image_scanner.get_scan_results_from_cache_async",
        side_effect=mock_get_image_cached_result_async,
    )
    mocker.patch(
        "checkov.common.images.image_referencer.get_license_statuses_async",
        side_effect=mock_get_license_statuses_async,
    )
    # 'workflow_with_string_container.yml (node:14.16 lines:12-13 (sha256:f9b91f78b0)).musl'
    reports = Runner().run(root_folder="", files=[str(test_file)], runner_filter=runner_filter)

    assert len(reports) == 2

    gha_report = next(report for report in reports if report.check_type == CheckType.GITHUB_ACTIONS)
    sca_image_report = next(report for report in reports if report.check_type == CheckType.SCA_IMAGE)

    assert len(gha_report.resources) == 0
    assert len(gha_report.passed_checks) == 14
    assert len(gha_report.failed_checks) == 2
    assert len(gha_report.skipped_checks) == 0
    assert len(gha_report.parsing_errors) == 0

    assert len(sca_image_report.resources) == 3
    assert sca_image_report.resources == {
        f".github/workflows/{file_name} ({image_name} lines:{code_lines} ({image_id})).{postfix}"
        for postfix in image_resource_postfixes
    }
    assert sca_image_report.image_cached_results[0]["dockerImageName"] == "node:14.16"
    assert sca_image_report.image_cached_results[0]["packages"] == [
        {"type": "os", "name": "tzdata", "version": "2021a-1+deb11u5", "licenses": []}
    ]

    assert len(sca_image_report.passed_checks) == 1
    assert len(sca_image_report.failed_checks) == 4
    assert len(sca_image_report.image_cached_results) == 1
    assert len(sca_image_report.skipped_checks) == 0
    assert len(sca_image_report.parsing_errors) == 0



import asyncio
from pathlib import Path
import responses
from checkov.common.util.tqdm_utils import ProgressBar

from checkov.sca_package.scanner import Scanner

EXAMPLES_DIR = Path(__file__).parent / "examples"


@responses.activate
def test_run_scan(mock_bc_integration, scan_result2, scan_result_success_response):
    # given
    responses.add(
        method=responses.POST,
        url=mock_bc_integration.bc_api_url + "/api/v1/vulnerabilities/scan",
        json={'id': '2e97f5afea42664309f492a1e2083b43479c2935', 'status': 'running'},
        status=202,
    )

    responses.add(
        method=responses.GET,
        url=mock_bc_integration.bc_api_url + "/api/v1/vulnerabilities/scan-results/"
                                             "2e97f5afea42664309f492a1e2083b43479c2935",
        json=scan_result_success_response,
        status=200
    )

    # when
    pbar = ProgressBar('')
    pbar.turn_off_progress_bar()
    scanner = Scanner(pbar)
    result = asyncio.run(scanner.run_scan((Path(EXAMPLES_DIR / "requirements.txt"))))

    # then
    assert len(result) == len(scan_result2)
    assert result.keys() == scan_result2.keys()
    assert len(result.get("packages")) == len(scan_result2.get("packages"))
    result_vuln_len = len(result.get("vulnerabilities"))
    scan_result_vuln_len = len(scan_result2.get("vulnerabilities"))
    assert result_vuln_len == scan_result_vuln_len
    assert sorted([result.get("vulnerabilities")[i]["id"] for i in range(result_vuln_len)]) == \
        sorted([scan_result2.get("vulnerabilities")[i]["id"] for i in range(scan_result_vuln_len)])
    assert result.get("complianceDistribution") == scan_result2.get("complianceDistribution")
    assert result.get("vulnerabilityDistribution") == scan_result2.get("vulnerabilityDistribution")
    responses.assert_call_count(mock_bc_integration.bc_api_url + "/api/v1/vulnerabilities/scan", 1)
    assert len(responses.calls) >= 2


@responses.activate
def test_run_scan_fail_on_scan(mock_bc_integration):
    # given
    responses.add(
        method=responses.POST,
        url=mock_bc_integration.bc_api_url + "/api/v1/vulnerabilities/scan",
        json={'id': '2e97f5afea42664309f492a1e2083b43479c2936', 'status': 'running'},
        status=202,
    )

    responses.add(
        method=responses.GET,
        url=mock_bc_integration.bc_api_url + "/api/v1/vulnerabilities/scan-results/"
                                             "2e97f5afea42664309f492a1e2083b43479c2936",
        json={
            "outputType": "Error",
            "outputData": "error_message"
        },
        status=400,
    )

    # when
    pbar = ProgressBar('')
    pbar.turn_off_progress_bar()
    result = asyncio.run(Scanner(pbar).run_scan(input_path=Path(EXAMPLES_DIR / "requirements.txt")))

    # then
    assert result == {}
    responses.assert_call_count(mock_bc_integration.bc_api_url + "/api/v1/vulnerabilities/scan", 1)
    assert len(responses.calls) >= 2



from textwrap import dedent

from checkov.common.output.ai import OpenAi


def test_parse_completion_response():
    # given
    content = dedent(
        """\
        The given code creates an AWS RDS instance with MySQL engine version 5.7, with a 10GB allocated storage, and a retention period of 7 days. However, it does not specify any encryption settings for the RDS instance, which violates the checkov policy 'Ensure all data stored in the RDS is securely encrypted at rest'. This means that sensitive data stored in the RDS instance may be vulnerable to unauthorized access or theft.
        
        To fix this, we need to enable encryption for the RDS instance. AWS RDS provides two options for encryption: AWS managed encryption and customer-managed encryption. In this case, we will use AWS managed encryption, which is the simpler option.
        
        To enable AWS managed encryption, we need to add the following block to the resource definition:
        
        ```
          storage_encrypted = true
          kms_key_id        = "alias/aws/rds"
        ```
        
        The `storage_encrypted` parameter enables encryption for the RDS instance, and the `kms_key_id` parameter specifies the AWS KMS key to use for encryption. In this case, we are using the default AWS RDS KMS key.
        
        The updated code with encryption enabled will look like this:
        
        ```
        resource "aws_db_instance" "pass" {
          allocated_storage      = 10
          db_name                = "mydb"
          engine                 = "mysql"
          engine_version         = "5.7"
          instance_class         = "db.t3.micro"
          username               = "foo"
          password               = "foobarbaz"
          parameter_group_name   = "default.mysql5.7"
          skip_final_snapshot    = true
          copy_tags_to_snapshot  = true
          backup_retention_period = 7
        
          storage_encrypted = true
          kms_key_id        = "alias/aws/rds"
        }
        ``` 
        
        With this change, the RDS instance will be encrypted at rest, and will comply with the checkov policy.
        """
    )

    # when
    details = OpenAi()._parse_completion_response(completion_content=content)

    # then
    assert details == [
        "The following text is AI generated and should be treated as a suggestion.",
        "",
        "The given code creates an AWS RDS instance with MySQL engine version 5.7, with a 10GB allocated storage, and a retention period of 7 days.",
        "However, it does not specify any encryption settings for the RDS instance, which violates the checkov policy 'Ensure all data stored in the RDS is securely encrypted at rest'.",
        "This means that sensitive data stored in the RDS instance may be vulnerable to unauthorized access or theft.",
        "",
        "To fix this, we need to enable encryption for the RDS instance.",
        "AWS RDS provides two options for encryption: AWS managed encryption and customer-managed encryption.",
        "In this case, we will use AWS managed encryption, which is the simpler option.",
        "",
        "To enable AWS managed encryption, we need to add the following block to the resource definition:",
        "",
        "  storage_encrypted = true",
        '  kms_key_id        = "alias/aws/rds"',
        "",
        "The `storage_encrypted` parameter enables encryption for the RDS instance, and the `kms_key_id` parameter specifies the AWS KMS key to use for encryption.",
        "In this case, we are using the default AWS RDS KMS key.",
        "",
        "The updated code with encryption enabled will look like this:",
        "",
        'resource "aws_db_instance" "pass" {',
        "  allocated_storage      = 10",
        '  db_name                = "mydb"',
        '  engine                 = "mysql"',
        '  engine_version         = "5.7"',
        '  instance_class         = "db.t3.micro"',
        '  username               = "foo"',
        '  password               = "foobarbaz"',
        '  parameter_group_name   = "default.mysql5.7"',
        "  skip_final_snapshot    = true",
        "  copy_tags_to_snapshot  = true",
        "  backup_retention_period = 7",
        "",
        "  storage_encrypted = true",
        '  kms_key_id        = "alias/aws/rds"',
        "}",
        "",
        "With this change, the RDS instance will be encrypted at rest, and will comply with the checkov policy.",
    ]



import unittest

from checkov.common.models.enums import CheckResult
from checkov.terraform.checks.resource.aws.EC2Credentials import check


class TestEC2Credentials(unittest.TestCase):
    def test_success(self):
        conf = {
            "ami": ["ami-04169656fea786776"],
            "instance_type": ["t2.nano"],
            "user_data": [
                '#! /bin/bash\nsudo apt-get update\nsudo apt-get install -y apache2\nsudo systemctl start apache2\nsudo systemctl enable apache2\nexport AWS_ACCESS_KEY_ID\nexport AWS_ACCESS_KEY_ID=FOO\nexport AWS_SECRET_ACCESS_KEY=bar\nexport AWS_DEFAULT_REGION=us-west-2\necho "<h1>Deployed via Terraform</h1>" | sudo tee /var/www/html/index.html'
            ],
            "tags": [{"Name": "${local.resource_prefix.value}-ec2"}],
        }

        scan_result = check.scan_resource_conf(conf=conf)
        self.assertEqual(CheckResult.PASSED, scan_result)
        conf = {"ami": ["ami-04169656fea786776"], "instance_type": ["t2.nano"]}
        scan_result = check.scan_resource_conf(conf=conf)

        self.assertEqual(CheckResult.PASSED, scan_result)

    def test_failure(self):
        conf = {
            "ami": ["ami-04169656fea786776"],
            "instance_type": ["t2.nano"],
            "user_data": [
                '#! /bin/bash\nsudo apt-get update\nsudo apt-get install -y apache2\nsudo systemctl start apache2\nsudo systemctl enable apache2\nexport AWS_ACCESS_KEY_ID\nexport AWS_ACCESS_KEY_ID=AKIAIOSFODNN7EXAMPLE\nexport AWS_SECRET_ACCESS_KEY=wJalrXUtnFEMI/K7MDENG/bPxRfiCYEXAMPLEKEY\nexport AWS_DEFAULT_REGION=us-west-2\necho "<h1>Deployed via Terraform</h1>" | sudo tee /var/www/html/index.html'
            ],
            "tags": [{"Name": "${local.resource_prefix.value}-ec2"}],
        }
        scan_result = check.scan_resource_conf(conf=conf)
        self.assertEqual(CheckResult.FAILED, scan_result)


if __name__ == "__main__":
    unittest.main()



import os

from parameterized import parameterized_class

from tests.terraform.graph.checks_infra.test_base import TestBaseSolver

TEST_DIRNAME = os.path.dirname(os.path.realpath(__file__))


@parameterized_class([
   {"graph_framework": "NETWORKX"},
   {"graph_framework": "IGRAPH"}
])
class TestRegexMatchSolver(TestBaseSolver):
    def setUp(self):
        self.checks_dir = TEST_DIRNAME
        super(TestRegexMatchSolver, self).setUp()

    def test_regex_match_solver_simple(self):
        root_folder = '../../../resources/tag_includes'
        check_id = "TagPrefix"
        should_pass = ['aws_subnet.acme_subnet']
        should_fail = ['aws_instance.some_instance']
        expected_results = {check_id: {"should_pass": should_pass, "should_fail": should_fail}}

        self.run_test(root_folder=root_folder, expected_results=expected_results, check_id=check_id)



import unittest
from pathlib import Path

from checkov.kubernetes.checks.resource.k8s.KubeControllerManagerBlockProfiles import check
from checkov.kubernetes.runner import Runner
from checkov.runner_filter import RunnerFilter


class TestKubeControllerManagerBlockProfiles(unittest.TestCase):
    def test_summary(self):
        # given
        test_files_dir = Path(__file__).parent / "example_KubeControllerManagerBlockProfiles"

        # when
        report = Runner().run(root_folder=str(test_files_dir), runner_filter=RunnerFilter(checks=[check.id]))

        # then
        summary = report.get_summary()

        passing_resources = {
            "Pod.kube-system.kube-controller-manager-disabled",
        }
        failing_resources = {
            "Pod.kube-system.kube-controller-manager-default",
            "Pod.kube-system.kube-controller-manager-enabled",
        }

        passed_check_resources = {c.resource for c in report.passed_checks}
        failed_check_resources = {c.resource for c in report.failed_checks}

        self.assertEqual(1, summary["passed"])
        self.assertEqual(2, summary["failed"])
        self.assertEqual(0, summary["skipped"])
        self.assertEqual(0, summary["parsing_errors"])

        self.assertEqual(passing_resources, passed_check_resources)
        self.assertEqual(failing_resources, failed_check_resources)


if __name__ == "__main__":
    unittest.main()



import unittest
from pathlib import Path

from checkov.runner_filter import RunnerFilter
from checkov.terraform.checks.resource.gcp.CloudArmorWAFACLCVE202144228 import check
from checkov.terraform.runner import Runner


class TestCloudArmorWAFACLCVE202144228(unittest.TestCase):
    def test(self):
        # given
        test_files_dir = Path(__file__).parent / "example_CloudArmorWAFACLCVE202144228"

        # when
        report = Runner().run(root_folder=str(test_files_dir), runner_filter=RunnerFilter(checks=[check.id]))

        # then
        summary = report.get_summary()

        passing_resources = {
            "google_compute_security_policy.enabled_deny_403",
            "google_compute_security_policy.enabled_deny_404",
        }

        failing_resources = {
            "google_compute_security_policy.allow",
            "google_compute_security_policy.preview",
            "google_compute_security_policy.different_expr",
        }

        passed_check_resources = {c.resource for c in report.passed_checks}
        failed_check_resources = {c.resource for c in report.failed_checks}

        self.assertEqual(summary["passed"], 2)
        self.assertEqual(summary["failed"], 3)
        self.assertEqual(summary["skipped"], 0)
        self.assertEqual(summary["parsing_errors"], 0)

        self.assertEqual(passing_resources, passed_check_resources)
        self.assertEqual(failing_resources, failed_check_resources)

        # check especially for the evaluated keys
        actual_evaluated_keys = next(
            c.check_result["evaluated_keys"]
            for c in report.failed_checks
            if c.resource == "google_compute_security_policy.different_expr"
        )
        expected_evaluated_keys = [
            "rule/[0]/action",
            "rule/[0]/preview",
            "rule/[0]/match/[0]/expr/[0]/expression",
        ]
        self.assertCountEqual(expected_evaluated_keys, actual_evaluated_keys)


if __name__ == "__main__":
    unittest.main()



from checkov.common.images.image_referencer import Image
from checkov.gitlab_ci.image_referencer.manager import GitlabCiImageReferencerManager


def test_extract_images_from_workflow():
    file_path = 'tests/gitlab_ci/resources/images/.gitlab-ci.yml'
    workflow_config = {
        "default": {
            "image": {
                "name": "ruby:2.6",
                "entrypoint": [
                    "/bin/bash"
                ],
                "__startline__": 3,
                "__endline__": 6
            },
            "services": [
                {
                    "name": "privateregistry/stuff/my-postgres:11.7",
                    "alias": "db-postgres",
                    "__startline__": 7,
                    "__endline__": 9
                },
                {
                    "name": "redis:latest",
                    "__startline__": 9,
                    "__endline__": 10
                },
                "nginx:1.17"
            ],
            "before_script": [
                "bundle install"
            ],
            "__startline__": 2,
            "__endline__": 15
        },
        "__startline__": 1,
        "__endline__": 32
    }

    manager = GitlabCiImageReferencerManager(workflow_config=workflow_config, file_path=file_path)
    images = manager.extract_images_from_workflow()

    assert set(images) == {
        Image(
            end_line=6,
            start_line=3,
            file_path=file_path,
            name='ruby:2.6',
            related_resource_id='default.image'
        ),
        Image(
            end_line=10,
            start_line=9,
            file_path=file_path,
            name='redis:latest',
            related_resource_id='default.services.2'
        ),
        Image(
            end_line=10,
            start_line=9,
            file_path=file_path,
            name='nginx:1.17',
            related_resource_id='default.services.2'
        ),
        Image(
            end_line=9,
            start_line=7,
            file_path=file_path,
            name='privateregistry/stuff/my-postgres:11.7',
            related_resource_id='default.services.1'
        )
    }


def test_extract_images_from_workflow_no_images():
    file_path = 'tests/gitlab_ci/resources/rules/.gitlab-ci.yml'
    workflow_config = {
        "script": "echo \"This job creates double pipelines!\"",
        "rules": [
            {
                "changes": [
                    "$DOCKERFILES_DIR/*"
                ],
                "__startline__": 4,
                "__endline__": 6
            },
            {
                "if": "$CI_PIPELINE_SOURCE \u003d\u003d \"push\"",
                "__startline__": 6,
                "__endline__": 7
            },
            {
                "if": "$CI_PIPELINE_SOURCE \u003d\u003d \"merge_request_event\"",
                "__startline__": 7,
                "__endline__": 9
            }
        ],
        "__startline__": 2,
        "__endline__": 9
    }

    manager = GitlabCiImageReferencerManager(workflow_config=workflow_config, file_path=file_path)
    images = manager.extract_images_from_workflow()

    assert not images




from unittest import mock

import igraph
import pytest
from networkx import DiGraph

from checkov.cloudformation.image_referencer.provider.aws import AwsCloudFormationProvider
from checkov.common.graph.graph_builder import CustomAttributes
from checkov.common.images.image_referencer import Image


@pytest.mark.parametrize("graph_framework", ['NETWORKX', 'IGRAPH'])
def test_extract_images_from_resources(graph_framework):
    # given
    resource = {
        "file_path_": "/ecs.yaml",
        "__endline__": 37,
        "__startline__": 5,
        "ContainerDefinitions": [
            {
                "Name": "my-app",
                "Image": "amazon/amazon-ecs-sample",
                "Cpu": 256,
                "EntryPoint": ["/usr/sbin/apache2", "-D", "FOREGROUND"],
                "Memory": 512,
                "Essential": True,
            },
            {
                "Name": "busybox",
                "Image": "busybox",
                "Cpu": 256,
                "EntryPoint": ["sh", "-c"],
                "Memory": 512,
                "Command": ['/bin/sh -c "while true; do /bin/date > /var/www/my-vol/date; sleep 1; done"'],
                "Essential": False,
            },
        ],
        "resource_type": "AWS::ECS::TaskDefinition",
    }
    if graph_framework == 'IGRAPH':
        graph = igraph.Graph()
        attr = resource
        graph.add_vertex(
            name='1',
            block_type_='resource',
            resource_type=attr[CustomAttributes.RESOURCE_TYPE] if CustomAttributes.RESOURCE_TYPE in attr else None,
            attr=attr,
        )
    else:
        graph = DiGraph()
        graph.add_node(1, **resource)

    # when
    with mock.patch.dict('os.environ', {'CHECKOV_GRAPH_FRAMEWORK': graph_framework}):
        aws_provider = AwsCloudFormationProvider(graph_connector=graph)
        images = aws_provider.extract_images_from_resources()

    # then
    assert images == [
        Image(
            file_path="/ecs.yaml",
            name="amazon/amazon-ecs-sample",
            start_line=5,
            end_line=37,
            related_resource_id="/ecs.yaml:None",
        ),
        Image(file_path="/ecs.yaml", name="busybox", start_line=5, end_line=37, related_resource_id="/ecs.yaml:None"),
    ]


@pytest.mark.parametrize("graph_framework", ['NETWORKX', 'IGRAPH'])
def test_extract_images_from_resources_with_no_image(graph_framework):
    # given
    resource = {
        "file_path_": "/ecs.yaml",
        "__endline__": 37,
        "__startline__": 5,
        "ContainerDefinitions": [
            {
                "Name": "my-app",
                "Cpu": 256,
                "EntryPoint": ["/usr/sbin/apache2", "-D", "FOREGROUND"],
                "Memory": 512,
                "Essential": True,
            },
        ],
        "resource_type": "AWS::ECS::TaskDefinition",
    }
    if graph_framework == 'IGRAPH':
        graph = igraph.Graph()
        attr = resource
        graph.add_vertex(
            name='1',
            block_type_='resource',
            resource_type=attr[CustomAttributes.RESOURCE_TYPE] if CustomAttributes.RESOURCE_TYPE in attr else None,
            attr=attr,
        )
    else:
        graph = DiGraph()
        graph.add_node(1, **resource)

    # when
    with mock.patch.dict('os.environ', {'CHECKOV_GRAPH_FRAMEWORK': graph_framework}):
        aws_provider = AwsCloudFormationProvider(graph_connector=graph)
        images = aws_provider.extract_images_from_resources()

    # then
    assert not images




import responses
import pytest
from unittest import mock

from checkov.terraform.module_loading.loaders.registry_loader import RegistryLoader
from checkov.terraform.module_loading.module_params import ModuleParams


@responses.activate
def test_module_version_url_invoked_once():
    # given
    module_version_url = "https://registry.terraform.io/v1/modules/terraform-aws-modules/example/versions"
    responses.add(
        method=responses.GET,
        url=module_version_url,
        json={"modules": [{"versions": [{"version": "1.0.0"}]}]},
        status=200,
    )
    loader = RegistryLoader()
    RegistryLoader.modules_versions_cache = {}  # reset cache
    module_params = ModuleParams("", "", "terraform-aws-modules/example", "", "", "")
    loader.discover(module_params)

    # when
    loader._is_matching_loader(module_params)
    loader._is_matching_loader(module_params)

    # then
    responses.assert_call_count(module_version_url, 1)
    assert loader.modules_versions_cache == {module_version_url: ["1.0.0"]}

def test_determine_tf_api_endpoints_tfc():
    # given
    loader = RegistryLoader()
    module_params = ModuleParams("", "", "terraform-aws-modules/example", "", "", "")
    loader.discover(module_params)

    # when
    loader._determine_tf_api_endpoints(module_params)

    # then
    assert module_params.tf_host_name == "app.terraform.io"
    assert module_params.tf_modules_endpoint == "https://registry.terraform.io/v1/modules"
    assert module_params.tf_modules_versions_endpoint == "https://registry.terraform.io/v1/modules/terraform-aws-modules/example/versions"

@responses.activate
def test_determine_tf_api_endpoints_tfe():
    # given
    loader = RegistryLoader()
    module_params = ModuleParams("", "", "example.registry.com/terraform-aws-modules/example", "", "", "")
    with mock.patch.dict("os.environ", {"TF_HOST_NAME": "example.registry.com"}):
        loader.discover(module_params)
    responses.add(
        method=responses.GET,
        url=f"https://{module_params.tf_host_name}/.well-known/terraform.json",
        json={
        "modules.v1": "/api/registry/v1/modules/",
        "providers.v1": "/api/registry/v1/providers/",
        "state.v2": "/api/v2/",
        "tfe.v2": "/api/v2/",
        "tfe.v2.1": "/api/v2/",
        "tfe.v2.2": "/api/v2/",
        "versions.v1": "https://checkpoint-api.hashicorp.com/v1/versions/"
        },
        status=200,
    )

    # when
    loader._determine_tf_api_endpoints(module_params)

    # then
    responses.assert_call_count(f"https://{module_params.tf_host_name}/.well-known/terraform.json", 1)
    assert module_params.tf_host_name == "example.registry.com"
    assert module_params.tf_modules_endpoint == "https://example.registry.com/api/registry/v1/modules/"
    assert module_params.tf_modules_versions_endpoint == "https://example.registry.com/api/registry/v1/modules//terraform-aws-modules/example/versions"

@pytest.mark.parametrize(
    "download_url, expected_result",
    [
        ("https://example.com/download?archive=tgz", True),
        ("https://example.com/download?archive=zip", True),
        ("https://example.com/download/module.zip", True),
        ("https://example.com/download/module/archive", False),
    ]
)
def test_is_download_url_archive(download_url, expected_result):
    is_archive = RegistryLoader._is_download_url_archive(download_url)
    assert is_archive == expected_result

@pytest.mark.parametrize(
    "tf_host_name, module_download_url, expected_result",
    [
        ("example.com", "https://example.com/download?archive=tgz", "https://example.com/download?archive=tgz"),
        ("example.com", "https://example.com/abc", "https://example.com/abc"),
        ("example.com", "/api/registry/v1/modules/namespace/version/download?archive=tgz", "https://example.com/api/registry/v1/modules/namespace/version/download?archive=tgz"),
    ]
)
def test_normalize_module_download_url(tf_host_name, module_download_url, expected_result):
    # given
    loader = RegistryLoader()
    module_params = ModuleParams("", "", "example.com/terraform-aws-modules/example", "", "", "")
    with mock.patch.dict("os.environ", {"TF_HOST_NAME": tf_host_name}):
        loader.discover(module_params)

    # when
    normalized_url = loader._normalize_module_download_url(module_params, module_download_url)

    # then
    assert normalized_url == expected_result

@pytest.mark.parametrize(
    "source_url",
    [
        ("git::https://example.com/repo.git"),
        ("git@github.com:org/repo"),
        ("github.com/org/repo"),
        ("bitbucket.org/org/repo"),
    ]
)
def test_is_matching_loader_git_sources(source_url):
    #given
    loader = RegistryLoader()
    module_params = ModuleParams("", "", source_url, "", "", "")
    loader.discover(module_params)

    # then
    assert not loader._is_matching_loader(module_params)



import unittest

import hcl2

from checkov.terraform.checks.resource.azure.SQLServerNoPublicAccess import check
from checkov.common.models.enums import CheckResult


class TestSQLServerNoPublicAccess(unittest.TestCase):

    def test_failure(self):
        hcl_res = hcl2.loads("""
            resource "azurerm_mysql_firewall_rule" "example" {
              name                = "office"
              resource_group_name = azurerm_resource_group.example.name
              server_name         = azurerm_mysql_server.example.name
              start_ip_address    = "0.0.0.0"
              end_ip_address      = "255.255.255.255"
            }
                """)
        resource_conf = hcl_res['resource'][0]['azurerm_mysql_firewall_rule']['example']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_success_allow_azure_services(self):
        hcl_res = hcl2.loads("""
            resource "azurerm_mysql_firewall_rule" "example" {
              name                = "office"
              resource_group_name = azurerm_resource_group.example.name
              server_name         = azurerm_mysql_server.example.name
              start_ip_address    = "0.0.0.0"
              end_ip_address      = "0.0.0.0"
            }
                """)
        resource_conf = hcl_res['resource'][0]['azurerm_mysql_firewall_rule']['example']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)

    def test_success(self):
        hcl_res = hcl2.loads("""
            resource "azurerm_mysql_firewall_rule" "example" {
              name                = "office"
              resource_group_name = azurerm_resource_group.example.name
              server_name         = azurerm_mysql_server.example.name
              start_ip_address    = "40.112.8.12"
              end_ip_address      = "40.112.8.17"
            }
                """)
        resource_conf = hcl_res['resource'][0]['azurerm_mysql_firewall_rule']['example']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)


if __name__ == '__main__':
    unittest.main()



import pytest

from checkov.circleci_pipelines.image_referencer.provider import CircleCIProvider
from checkov.circleci_pipelines.runner import Runner


def test_extract_images_from_workflow(circle_ci_filepath_workflow_with_images_config,
                                      circle_ci_image1, circle_ci_image2):
    file_path, config = circle_ci_filepath_workflow_with_images_config

    provider = CircleCIProvider(file_path=file_path, workflow_config=config)
    images = provider.extract_images_from_workflow()

    assert set(images) == {circle_ci_image1, circle_ci_image2}


def test_extract_images_from_workflow_no_images(circle_ci_filepath_workflow_no_images_config):
    file_path, config = circle_ci_filepath_workflow_no_images_config

    provider = CircleCIProvider(file_path=file_path, workflow_config=config)
    images = provider.extract_images_from_workflow()

    assert not images

@pytest.mark.parametrize(
    "start_line, end_line, tag, supported_entities, old_key_format, expected_key",
    [
        (21,
         22,
         'jobs',
         ('jobs.*.docker[].{image: image, __startline__: __startline__, __endline__:__endline__}',),
         'jobs.*.docker[].{image: image, __startline__: __startline__, __endline__:__endline__}.jobs.*.docker[].{image: image, __startline__: __startline__, __endline__:__endline__}.CKV_CIRCLECIPIPELINES_1[85:86]',
         "jobs(test-docker-versioned-img).docker.image[1](mongo:2.6.8)"),
        (16,
         18,
         'executors',
         ('executors.*.docker[].{image: image, __startline__: __startline__, __endline__:__endline__}',),
         'executors.*.docker[].{image: image, __startline__: __startline__, __endline__:__endline__}.executors.*.docker[].{image: image, __startline__: __startline__, __endline__:__endline__}.CKV_CIRCLECIPIPELINES_8[16:18]',
         "executors(image-executor).docker.image[1](mongo:2.6.8)")
    ]
)
def test_generate_resource_key_generates_same_key_as_get_resource(file_path,
                                                                  start_line, end_line, tag,
                                                                  supported_entities,
                                                                  old_key_format, expected_key,
                                                                  circleci_config_with_images_definitions):
    definitions = circleci_config_with_images_definitions.get(file_path)
    provider = CircleCIProvider(definitions, file_path)
    runner = Runner()
    runner.definitions[file_path] = definitions

    key1 = runner.get_resource(file_path, old_key_format, supported_entities, start_line, end_line)
    key2 = provider.generate_resource_key(start_line, end_line, tag)

    assert key1 == key2 == expected_key



import unittest
from pathlib import Path

from checkov.runner_filter import RunnerFilter
from checkov.terraform.checks.resource.yandexcloud.ObjectStorageBucketPublicAccess import scanner
from checkov.terraform.runner import Runner

class TestObjectStorageBucketPublicAccess(unittest.TestCase):
    def test(self):
        test_files_dir = Path(__file__).parent / "example_ObjectStorageBucketPublicAccess"
        report = Runner().run(root_folder=str(test_files_dir), runner_filter=RunnerFilter(checks=[scanner.id]))
        summary = report.get_summary()

        passing_resources = {
            "yandex_storage_bucket.pass"
        }
        failing_resources = {
            "yandex_storage_bucket.fail-1",
            "yandex_storage_bucket.fail-2",
            "yandex_storage_bucket.fail-3"
        }

        passed_check_resources = {c.resource for c in report.passed_checks}
        failed_check_resources = {c.resource for c in report.failed_checks}

        self.assertEqual(summary["passed"], 1)
        self.assertEqual(summary["failed"], 3)
        self.assertEqual(summary["skipped"], 0)
        self.assertEqual(summary["parsing_errors"], 0)

        self.assertEqual(passing_resources, passed_check_resources)
        self.assertEqual(failing_resources, failed_check_resources)

if __name__ == "__main__":
    unittest.main()



import unittest

import hcl2

from checkov.terraform.checks.resource.gcp.GoogleComputeBlockProjectSSH import check
from checkov.common.models.enums import CheckResult


class TestGoogleComputeBlockProjectSSH(unittest.TestCase):

    def test_failure(self):
        hcl_res = hcl2.loads("""
            resource "google_compute_instance" "default" {
              name         = "test"
              machine_type = "n1-standard-1"
              zone         = "us-central1-a"
            }
                """)
        resource_conf = hcl_res['resource'][0]['google_compute_instance']['default']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_failure_1(self):
        hcl_res = hcl2.loads("""
            resource "google_compute_instance" "default" {
              name         = "test"
              machine_type = "n1-standard-1"
              zone         = "us-central1-a"
              metadata = {
                 block-project-ssh-keys = false
              }
            }
                """)
        resource_conf = hcl_res['resource'][0]['google_compute_instance']['default']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_failure_2(self):
        hcl_res = hcl2.loads("""
            resource "google_compute_instance_from_template" "default" {
              name         = "test"
              source_instance_template = google_compute_instance_template.default.id
              metadata {
                foo = "bar"
                hey = "oh"
                block-project-ssh-keys = false
                }
            }
                """)
        resource_conf = hcl_res['resource'][0]['google_compute_instance_from_template']['default']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_failure_3(self):
        hcl_res = hcl2.loads("""
            resource "google_compute_instance_template" "default_template" {
              name         = "test"
              machine_type = "e2-medium"

              disk {
                source_image = "debian-cloud/debian-9"
                auto_delete  = true
                disk_size_gb = 100
                boot         = true
              }

              network_interface {
                network = "default"
              }

              metadata = {
                foo = "bar"
              }

              can_ip_forward = true
            }
                """)
        resource_conf = hcl_res['resource'][0]['google_compute_instance_template']['default_template']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_unknown_1(self):
        hcl_res = hcl2.loads("""
            resource "google_compute_instance_from_template" "default" {
              name         = "test"
              source_instance_template = google_compute_instance_template.default.id
              metadata {
                foo = "bar"
                hey = "oh"
                }
            }
                """)
        resource_conf = hcl_res['resource'][0]['google_compute_instance_from_template']['default']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.UNKNOWN, scan_result)

    def test_unknown_2(self):
        hcl_res = hcl2.loads("""
            resource "google_compute_instance_from_template" "default" {
              name         = "test"
              source_instance_template = google_compute_instance_template.default.id
            }
                """)
        resource_conf = hcl_res['resource'][0]['google_compute_instance_from_template']['default']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.UNKNOWN, scan_result)

    def test_success_1(self):
        hcl_res = hcl2.loads("""
            resource "google_compute_instance" "default" {
              name         = "test"
              machine_type = "n1-standard-1"
              zone         = "us-central1-a"
              metadata = {
                 block-project-ssh-keys = true
              }
            }
                """)
        resource_conf = hcl_res['resource'][0]['google_compute_instance']['default']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)

    def test_success_2(self):
        hcl_res = hcl2.loads("""
            resource "google_compute_instance_template" "default_template" {
              name         = "test"
              machine_type = "n1-standard-1"
              zone         = "us-central1-a"
                              
              disk {
                source_image = "debian-cloud/debian-9"
                auto_delete  = true
                disk_size_gb = 100
                boot         = true
              }
            
              network_interface {
                network = "default"
              }
              
              can_ip_forward = true
              metadata = {
                foo                    = "bar",
                block-project-ssh-keys = true
              }
            }
                """)
        resource_conf = hcl_res['resource'][0]['google_compute_instance_template']['default_template']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)

    def test_success_3(self):
        hcl_res = hcl2.loads("""
            resource "google_compute_instance_from_template" "default" {
              name         = "test"
              source_instance_template = google_compute_instance_template.default.id
              metadata {
                foo = "bar"
                hey = "oh"
                block-project-ssh-keys = true
                }
            }
                """)
        resource_conf = hcl_res['resource'][0]['google_compute_instance_from_template']['default']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)


if __name__ == '__main__':
    unittest.main()



from checkov.common.util.dockerfile import is_docker_file

VALID_DOCKER_FILE_NAMES = ["Dockerfile", "dockerfile", "Dockerfile.prod", "Dockerfile.Product1", "dev.Dockerfile",
                           "team1.product.dockerfile"]
INVALID_DOCKER_FILE_NAMES = ["package.json", "dockerfil", "dockerfilee", ".dockerfile", "ddockerfile",
                             "ockerfile", "docker-file", "dockerfile1"]


def test_is_docker_file():
    assert all(is_docker_file(curr_name) for curr_name in VALID_DOCKER_FILE_NAMES)
    assert all(not is_docker_file(curr_name) for curr_name in INVALID_DOCKER_FILE_NAMES)



import unittest

import hcl2

from checkov.terraform.checks.resource.gcp.GoogleProjectMemberDefaultServiceAccount import check
from checkov.common.models.enums import CheckResult


class TestGoogleProjectMemberDefaultServiceAccount(unittest.TestCase):

    def test_failure_binding(self):
        hcl_res = hcl2.loads("""
            resource "google_project_iam_binding" "project" {
              project = "your-project-id"
              role    = "roles/editor"
            
              members = [
                "user",
                "serviceAccount:test-compute@developer.gserviceaccount.com",
              ]
            }
                """)
        resource_conf = hcl_res['resource'][0]['google_project_iam_binding']['project']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_failure_member(self):
        hcl_res = hcl2.loads("""
            resource "google_project_iam_member" "project" {
              project = "your-project-id"
              role    = "roles/editor"
              member  = "serviceAccount:test-compute@developer.gserviceaccount.com"
            }
                """)
        resource_conf = hcl_res['resource'][0]['google_project_iam_member']['project']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_success_binding(self):
        hcl_res = hcl2.loads("""
            resource "google_project_iam_binding" "project" {
              project = "your-project-id"
              role    = "roles/editor"
            
              members = [
                "user@mail.com",
              ]
            }
                """)
        resource_conf = hcl_res['resource'][0]['google_project_iam_binding']['project']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)

    def test_success_member(self):
        hcl_res = hcl2.loads("""
            resource "google_project_iam_member" "project" {
              project = "your-project-id"
              role    = "roles/editor"
              member  = "user@mail.com"
            }
                """)
        resource_conf = hcl_res['resource'][0]['google_project_iam_member']['project']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)



if __name__ == '__main__':
    unittest.main()



import os
import unittest

from checkov.common.bridgecrew.severities import Severities, BcSeverities
from checkov.common.output.report import CheckType
from checkov.runner_filter import RunnerFilter
from checkov.helm.runner import Runner
from tests.helm.utils import helm_exists


class TestRunnerValid(unittest.TestCase):
    @unittest.skipIf(not helm_exists(), "helm not installed")
    def test_record_relative_path_with_relative_dir(self):
        # test whether the record's repo_file_path is correct, relative to the CWD (with a / at the start).

        # this is just constructing the scan dir as normal
        current_dir = os.path.dirname(os.path.realpath(__file__))
        scan_dir_path = os.path.join(current_dir, "runner", "resources", "infrastructure")

        # this is the relative path to the directory to scan (what would actually get passed to the -d arg)
        dir_rel_path = os.path.relpath(scan_dir_path).replace("\\", "/")

        checks_allowlist = ["CKV_K8S_42"]

        runner = Runner()
        report = runner.run(
            root_folder=dir_rel_path, runner_filter=RunnerFilter(framework=["helm"], checks=checks_allowlist)
        )

        all_checks = report.failed_checks + report.passed_checks
        self.assertEqual(len(report.passed_checks), 0)
        self.assertEqual(len(report.failed_checks), 1)
        self.assertEqual(report.check_type, CheckType.HELM)
        for record in all_checks:
            self.assertIn(record.repo_file_path, record.file_path)
        for resource in report.resources:
            self.assertIn('/helm-tiller/pwnchart/templates', resource)

    @unittest.skipIf(not helm_exists(), "helm not installed")
    def test_runner_honors_enforcement_rules(self):
        current_dir = os.path.dirname(os.path.realpath(__file__))
        scan_dir_path = os.path.join(current_dir, "runner", "resources", "infrastructure")

        runner = Runner()
        filter = RunnerFilter(framework=['helm'], use_enforcement_rules=True)
        # this is not quite a true test, because the checks don't have severities. However, this shows that the check registry
        # passes the report type properly to RunnerFilter.should_run_check, and we have tests for that method
        filter.enforcement_rule_configs = {CheckType.HELM: Severities[BcSeverities.OFF]}
        report = runner.run(
            root_folder=scan_dir_path, runner_filter=filter
        )

        self.assertEqual(len(report.failed_checks), 0)
        self.assertEqual(len(report.passed_checks), 0)
        self.assertEqual(len(report.skipped_checks), 0)
        self.assertEqual(len(report.parsing_errors), 0)
        
    @unittest.skipIf(not helm_exists(), "helm not installed")
    def test_runner_invalid_chart(self):
        current_dir = os.path.dirname(os.path.realpath(__file__))
        scan_dir_path = os.path.join(current_dir, "runner", "resources", "schema-registry")

        runner = Runner()
        filter = RunnerFilter(framework=['helm'], use_enforcement_rules=False)
        # this is not quite a true test, because the checks don't have severities. However, this shows that the check registry
        # passes the report type properly to RunnerFilter.should_run_check, and we have tests for that method
        report = runner.run(
            root_folder=scan_dir_path, runner_filter=filter
        )

        self.assertEqual(len(report.failed_checks), 0)


if __name__ == "__main__":
    unittest.main()



import unittest

import hcl2

from checkov.terraform.checks.resource.azure.AzureManagedDiskEncryptionSet import check
from checkov.common.models.enums import CheckResult


class TestAzureManagedDiskEncryptionSet(unittest.TestCase):

    def test_failure(self):
        hcl_res = hcl2.loads("""
        resource "azurerm_managed_disk" "source" {
              name                 = "acctestmd1"
              location             = "West US 2"
              resource_group_name  = azurerm_resource_group.example.name
              storage_account_type = "Standard_LRS"
              create_option        = "Empty"
              disk_size_gb         = "1"
            
              tags = {
                environment = "staging"
              }
            }
            """)
        resource_conf = hcl_res['resource'][0]['azurerm_managed_disk']['source']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_success(self):
        hcl_res = hcl2.loads("""
        resource "azurerm_managed_disk" "source" {
              name                 = "acctestmd1"
              location             = "West US 2"
              resource_group_name  = azurerm_resource_group.example.name
              storage_account_type = "Standard_LRS"
              create_option        = "Empty"
              disk_size_gb         = "1"
              disk_encryption_set_id = "koko"
              tags = {
                environment = "staging"
              }
            }
        """)
        resource_conf = hcl_res['resource'][0]['azurerm_managed_disk']['source']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)


if __name__ == '__main__':
    unittest.main()



import os
import unittest

from parameterized import parameterized_class

from checkov.common.bridgecrew.check_type import CheckType
from checkov.common.bridgecrew.severities import Severities, BcSeverities
from checkov.common.graph.db_connectors.igraph.igraph_db_connector import IgraphConnector
from checkov.common.graph.db_connectors.networkx.networkx_db_connector import NetworkxConnector
from checkov.runner_filter import RunnerFilter
from checkov.yaml_doc.runner import Runner
from checkov.yaml_doc.registry import registry


@parameterized_class([
   {"db_connector": NetworkxConnector},
   {"db_connector": IgraphConnector}
])
class TestRunnerValid(unittest.TestCase):

    def test_registry_has_type(self):
        self.assertEqual(registry.report_type, CheckType.YAML)

    def test_runner_object_failing_check(self):
        current_dir = os.path.dirname(os.path.realpath(__file__))
        valid_dir_path = os.path.join(current_dir, "resources", "object", "fail")
        checks_dir = os.path.join(current_dir, "checks", "object")
        runner = Runner(db_connector=self.db_connector())
        checks = ["CKV_FOO_1", "CKV_FOO_2"]
        report = runner.run(
            root_folder=valid_dir_path,
            external_checks_dir=[checks_dir],
            runner_filter=RunnerFilter(framework='all', checks=checks)
        )
        self.assertEqual(len(report.failed_checks), 3)
        self.assertEqual(report.parsing_errors, [])
        self.assertEqual(len(report.passed_checks), 2)
        self.assertEqual(report.skipped_checks, [])
        report.print_console()

    def test_runner_honors_enforcement_rules(self):
        current_dir = os.path.dirname(os.path.realpath(__file__))
        valid_dir_path = os.path.join(current_dir, "resources", "object", "fail")
        checks_dir = os.path.join(current_dir, "checks", "object")
        runner = Runner(db_connector=self.db_connector())
        filter = RunnerFilter(framework=['yaml'], use_enforcement_rules=True)
        # this is not quite a true test, because the checks don't have severities. However, this shows that the check registry
        # passes the report type properly to RunnerFilter.should_run_check, and we have tests for that method
        filter.enforcement_rule_configs = {CheckType.YAML: Severities[BcSeverities.OFF]}
        report = runner.run(
            root_folder=valid_dir_path,
            external_checks_dir=[checks_dir],
            runner_filter=filter
        )
        self.assertEqual(len(report.failed_checks), 0)
        self.assertEqual(len(report.passed_checks), 0)
        self.assertEqual(len(report.skipped_checks), 0)
        self.assertEqual(len(report.parsing_errors), 0)
        report.print_console()

    def test_runner_object_passing_check(self):
        current_dir = os.path.dirname(os.path.realpath(__file__))
        valid_dir_path = os.path.join(current_dir, "resources", "object", "pass")
        checks_dir = os.path.join(current_dir, "checks", "object")
        runner = Runner(db_connector=self.db_connector())
        report = runner.run(
            root_folder=valid_dir_path,
            external_checks_dir=[checks_dir],
            runner_filter=RunnerFilter(framework="all", checks=["CKV_FOO_1"]),
        )
        self.assertEqual(len(report.passed_checks), 1)
        self.assertEqual(report.parsing_errors, [])
        self.assertEqual(report.failed_checks, [])
        self.assertEqual(report.skipped_checks, [])
        report.print_console()

    def test_runner_object_skip_check(self):
        current_dir = os.path.dirname(os.path.realpath(__file__))
        valid_dir_path = os.path.join(current_dir, "resources", "object", "skip")
        checks_dir = os.path.join(current_dir, "checks", "object")
        runner = Runner(db_connector=self.db_connector())
        report = runner.run(
            root_folder=valid_dir_path,
            external_checks_dir=[checks_dir],
            runner_filter=RunnerFilter(framework="all", checks=["CKV_FOO_1", "CKV_FOO_2"]),
        )
        self.assertEqual(len(report.passed_checks), 1)
        self.assertEqual(report.parsing_errors, [])
        self.assertEqual(report.failed_checks, [])
        self.assertEqual(len(report.skipped_checks), 1)
        report.print_console()

    def test_runner_array_failing_check(self):
        current_dir = os.path.dirname(os.path.realpath(__file__))
        valid_dir_path = os.path.join(current_dir, "resources", "array", "fail")
        checks_dir = os.path.join(current_dir, "checks", "array")
        runner = Runner(db_connector=self.db_connector())
        report = runner.run(
            root_folder=valid_dir_path,
            external_checks_dir=[checks_dir],
            runner_filter=RunnerFilter(framework='all', checks=["CKV_BARBAZ_1"])
        )
        self.assertEqual(len(report.failed_checks), 3)
        self.assertEqual(report.parsing_errors, [])
        self.assertEqual(len(report.passed_checks), 2)
        self.assertEqual(report.skipped_checks, [])
        report.print_console()

    def test_runner_array_passing_check(self):
        current_dir = os.path.dirname(os.path.realpath(__file__))
        valid_dir_path = os.path.join(current_dir, "resources", "array", "pass")
        checks_dir = os.path.join(current_dir, "checks", "array")
        runner = Runner(db_connector=self.db_connector())
        report = runner.run(
            root_folder=valid_dir_path,
            external_checks_dir=[checks_dir],
            runner_filter=RunnerFilter(framework="all", checks=["CKV_BARBAZ_1"]),
        )
        self.assertEqual(len(report.passed_checks), 3)
        self.assertEqual(report.parsing_errors, [])
        self.assertEqual(report.failed_checks, [])
        self.assertEqual(report.skipped_checks, [])
        report.print_console()

    def test_runner_complex_failing_check(self):
        current_dir = os.path.dirname(os.path.realpath(__file__))
        valid_dir_path = os.path.join(current_dir, "resources", "complex", "fail")
        checks_dir = os.path.join(current_dir, "checks", "complex")
        runner = Runner(db_connector=self.db_connector())
        report = runner.run(
            root_folder=valid_dir_path,
            external_checks_dir=[checks_dir],
            runner_filter=RunnerFilter(framework='all', checks=["CKV_COMPLEX_1"])
        )
        self.assertEqual(len(report.failed_checks), 1)
        self.assertEqual(report.parsing_errors, [])
        self.assertEqual(report.passed_checks, [])
        self.assertEqual(report.skipped_checks, [])
        report.print_console()

    def test_runner_complex_passing_check(self):
        current_dir = os.path.dirname(os.path.realpath(__file__))
        valid_dir_path = os.path.join(current_dir, "resources", "complex", "pass")
        checks_dir = os.path.join(current_dir, "checks", "complex")
        runner = Runner(db_connector=self.db_connector())
        report = runner.run(
            root_folder=valid_dir_path,
            external_checks_dir=[checks_dir],
            runner_filter=RunnerFilter(framework="all", checks=["CKV_COMPLEX_1"]),
        )
        self.assertEqual(len(report.passed_checks), 2)
        self.assertEqual(report.parsing_errors, [])
        self.assertEqual(report.failed_checks, [])
        self.assertEqual(report.skipped_checks, [])
        report.print_console()


if __name__ == "__main__":
    unittest.main()



import unittest
from checkov.terraform.parser import Parser
from checkov.terraform.context_parsers.registry import parser_registry
import os


class TestVariableContextParser(unittest.TestCase):
    def setUp(self):
        test_root_dir = os.path.dirname(os.path.realpath(__file__)) + '/../evaluation/resources/default_evaluation/'
        tf_definitions = {}
        parsing_errors = {}
        Parser().parse_directory(directory=test_root_dir,
                                  out_definitions=tf_definitions,
                                  out_parsing_errors=parsing_errors)
        for definition in tf_definitions.items():
            definitions_context = parser_registry.enrich_definitions_context(definition)
        self.definitions_context = definitions_context

    def test_assignments_exists(self):
        self.assertIsNotNone(
            self.definitions_context[os.path.dirname(
                os.path.realpath(__file__)) + '/../evaluation/resources/default_evaluation/variables.tf'][
                'variable'].get(
                'assignments'))

    def test_assignment_value(self):
        self.assertFalse(
            self.definitions_context[os.path.dirname(
                os.path.realpath(__file__)) + '/../evaluation/resources/default_evaluation/variables.tf'][
                'variable'].get(
                'assignments').get('user_exists')
        )
        self.assertEqual(
            self.definitions_context[os.path.dirname(
                os.path.realpath(__file__)) + '/../evaluation/resources/default_evaluation/variables.tf'][
                'variable'].get(
                'assignments').get('app_client_id'), 'Temp')


if __name__ == '__main__':
    unittest.main()
